package org.cl0zzzy.http;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.charset.StandardCharsets;
import java.nio.file.Paths;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.cl0zzzy.sqlite.Account;
import org.cl0zzzy.sqlite.DB;
import org.cl0zzzy.rtmp.Util;

import fi.iki.elonen.NanoHTTPD;

public class HttpMain extends NanoHTTPD {
	
	private int port;
	private int rtmp;
	private int count = 0;
	
	public HttpMain(int port, int rtmp) {
		super(port);
		System.out.println("HTTP daemon started");
		this.port = port;
		this.rtmp = rtmp;
		try {
			this.start();
		} catch( Exception e ) {
			e.printStackTrace();
		}
	}
	
    public void updatePlayers( int count ) {
    	this.count = count;
    }
	
	 @Override
     public Response serve(IHTTPSession session) {
		 String response = "<html><body><h1>404</h1></body></html>";
		 try {
			 Map<String, String> p = session.getParms();
			 if (session.getUri().startsWith("/png/")) {
				 try {
					 return parsePNG(session.getUri().substring(4));
				 } catch (Exception e) {
					 e.printStackTrace();
				 }
			 } else if (session.getUri().startsWith("/swf/")) {
				 try {
					 return parseSWF(session.getUri().substring(4));
				 } catch (Exception e) {
					 e.printStackTrace();
				 }
			 } else if (session.getUri().endsWith(".swf")) {
				 try {
					 return parseSWFN(session.getUri().substring(1));
				 } catch (Exception e) {
					 e.printStackTrace();
				 }
			 } else if (session.getUri().startsWith("/css/")) {
				 return parseCSS(session.getUri().substring(4));
			 } else if (session.getUri().startsWith("/async/")) {
				 if (session.getUri().equals("/async/ServerAction")) {
					 if (session.getCookies().read("UID") != null && session.getCookies().read("TOKEN") != null) {
						 if (Util.MD5(DB.getSecureCode() + session.getCookies().read("UID")).equals(session.getCookies().read("TOKEN"))) {
							 response = this.generateServerAction(Integer.parseInt(session.getCookies().read("UID")));
							 return new NanoHTTPD.Response(Response.Status.OK, "text/xml;charset=UTF-8", response);
						 }
					 }
					 response = "<?xml version=\"1.0\" encoding=\"utf-8\"?><response/>";
					 return new NanoHTTPD.Response(Response.Status.OK, "text/xml;charset=UTF-8", response);
				 } else if (session.getUri().equals("/async/Ping")) {
					 response = "<response isPong=\"true\" />";
					 return new NanoHTTPD.Response(Response.Status.OK, "text/xml;charset=UTF-8", response);
				 }
			 } else if (session.getUri().startsWith("/img/")) {
				 return parseImg(session.getUri().substring(4));
			 } else {
				 if (session.getCookies().read("UID") != null && session.getCookies().read("TOKEN") != null) {
					 if (Util.MD5(DB.getSecureCode() + session.getCookies().read("UID")).equals(session.getCookies().read("TOKEN"))) {
						 session.getCookies().set(new Cookie("UID", session.getCookies().read("UID")));
						 session.getCookies().set(new Cookie("TOKEN", Util.MD5(DB.getSecureCode() + session.getCookies().read("UID"))));
						 NanoHTTPD.Response nr = new NanoHTTPD.Response(parseMain().replaceAll("REPLACE_FOR_DOMAIN", "localhost:" + port));
						 return nr;
					 } else {
						 session.getCookies().delete("UID");
						 session.getCookies().delete("TOKEN");
					 }
				 }
				 Map<String, String> prms = new HashMap<String, String>();
				 try {
					 session.parseBody(prms);
				 } catch (IOException e) {
					 e.printStackTrace();
				 } catch (ResponseException e) {
					 e.printStackTrace();
				 }
				 p = session.getParms();
				 if (p.get("username") != null && p.get("password") != null) {
					 if (p.get("act") != null) {
						 String regex = "^[a-zA-z а-яА-Я0-9\\_\\-]{3,30}$";
						 Pattern pattern = Pattern.compile(regex);
						 Matcher matcher = pattern.matcher(p.get("username"));
						 if (matcher.matches() && DB.getUserAccountByName(p.get("username")) == null) {
							 DB.insertAccount(p.get("username"), p.get("password"));
							 response = parseLog();
							 return new NanoHTTPD.Response(response);
						 }
					 } else {
						 Account acc = DB.getUserAccountByName(p.get("username"));
						 if (acc != null && acc.getPassword().equals(Util.MD5(p.get("password")))) {
							 session.getCookies().set(new Cookie("TOKEN", Util.MD5(DB.getSecureCode() + acc.getId())));
							 session.getCookies().set(new Cookie("UID", acc.getId() + ""));
							 NanoHTTPD.Response nr = new NanoHTTPD.Response(parseMain().replaceAll("REPLACE_FOR_DOMAIN", "localhost:" + port));
							 return nr;
						 }
					 }
				 }
				 if (p.get("act") != null) {
					 response = parseReg();
				 } else {
					 response = parseLog();
				 }
			 }
		 } catch ( Exception e ) {
			 e.printStackTrace();
		 }
         return new NanoHTTPD.Response(response);
     }

	 private Response parsePNG(String png) throws Exception {
		return getRespFromIS("image/png", new FileInputStream(new File("png"+png)));
	}

	private Response parseSWF(String swf) throws Exception {
		return getRespFromIS("application/x-shockwave-flash", new FileInputStream(new File("swf"+swf)));
	}
	
	private Response parseSWFN(String swf) throws Exception {
		return getRespFromIS("application/x-shockwave-flash", new FileInputStream(new File(swf)));
	}

	private String parseMain() {
		 return getStringFromIS(getFileFromResourceAsStream("/res/main.html"));
	 }
	 
	 private String parseReg() {
		 return getStringFromIS(getFileFromResourceAsStream("/res/reg.html"));
	 }
	 
	 private String parseLog() {
		 return getStringFromIS(getFileFromResourceAsStream("/res/log.html"));
	 }
	 
	 private NanoHTTPD.Response parseImg( String filename ) {
		 return getRespFromIS("image/jpeg", (FileInputStream) getFileFromResourceAsStream("/res/img" + filename));
	 }
	 
	 private NanoHTTPD.Response parseCSS( String filename ) {
		 return getRespFromIS("text/css", (FileInputStream) getFileFromResourceAsStream("/res/css" + filename));
	 }
	 
	 private NanoHTTPD.Response getRespFromIS( String mime, FileInputStream fis ) {
		 NanoHTTPD.Response res = new NanoHTTPD.Response(Response.Status.OK, mime, fis);
		 return res;
	 }
	 
    private InputStream getFileFromResourceAsStream(String fileName) {
        try {
			return new FileInputStream(Paths.get(this.getClass().getResource(fileName).toURI()).toFile());
		} catch (Exception e) {
			e.printStackTrace();
		}
        return null;
    }
    
    private String getStringFromIS(InputStream is) {
    	String data = "";
        try (InputStreamReader streamReader =
                    new InputStreamReader(is, StandardCharsets.UTF_8);
             BufferedReader reader = new BufferedReader(streamReader)) {

            String line;
            while ((line = reader.readLine()) != null) {
                data += line;
            }

        } catch (IOException e) {
            e.printStackTrace();
        }
        return data;
    }
    
    private String generateServerAction( int UID ) {
    	String response = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\r\n" + 
    			"<response>\r\n" + 
    			"<sn_status IsBinded=\"0\" />\r\n" + 
    			"<user_name Value=\""+ DB.getPlayerName( UID ) +"\" />\r\n" + 
    			"<preloader>\r\n" + 
    				DB.getPreloaders()+
    			"</preloader>\r\n" + 
    			"<flags IsUserDetailsMissing=\"1\" EntranceCount=\"13\" />\r\n" + 
    			"<tutorial>\r\n" +
    			"		  <item Id=\"-1\" State=\"1\" />\r\n" + 
    			"		  <item Id=\"1\" State=\"1\" />\r\n" + 
    			"		  <item Id=\"2\" State=\"1\" />\r\n" + // 1
    			"		  <item Id=\"3\" State=\"1\" />\r\n" + 
    			"		  <item Id=\"4\" State=\"1\" />\r\n" + 
    			"		  <item Id=\"5\" State=\"1\" />" +
    			"</tutorial>\r\n" +
    			"<grants ReceivingCount=\"0\" />\r\n" + 
    			"<requests ReceivingCount=\"0\" />\r\n" + 
    			"<promotion>\r\n" + 
    				DB.getPromotions()+
    			"</promotion>\r\n" + 
    			"<promotion_banner>\r\n" + 
    				DB.getPromotionBanners()+
    			"</promotion_banner>\r\n" + 
    			"<phone>\r\n" + 
    			"	<messages>\r\n" + 
    			"		"+DB.getPhoneMessages(UID)+"\r\n" + 
    			"	</messages>\r\n" + 
    			"</phone>\r\n" + 
    			"<postcard>\r\n" + 
    			"	<messages>\r\n" + 
    			"		"+DB.getPostcards(UID)+"\r\n" + 
    			"	</messages>\r\n" + 
    			"</postcard>\r\n" + 
    			"<miniquest>\r\n" +
    			//"			<i Id=\"1000001\" MiniquestId=\"1000001\" IsPostponed=\"0\" StartDate=\"2019-01-11T21:32:57.883\">\r\n" + 
    			//"			<i Id=\"10000011\" TaskId=\"10000011\" IsFinished=\"0\" Counter=\"0\" /></i>" +
    			"</miniquest>\r\n" + 
    			"<cdata value=\"" +Util.bytesToHex(new RC4("_level0".getBytes()).encrypt("<config><item Id=\"1\" Parameter=\"IsPreloaderEnabled\" Value=\"1\" Type=\"bool\" /><item Id=\"2\" Parameter=\"SynchronizeAvatarRotation\" Value=\"1\" Type=\"bool\" /><item Id=\"3\" Parameter=\"InitialVolumeValue\" Value=\"30\" Type=\"number\" /><item Id=\"4\" Parameter=\"IsStartupHomeLocation\" Value=\"0\" Type=\"bool\" /><item Id=\"5\" Parameter=\"AccessRoleFlags\" Value=\"0\" Type=\"number\" /><item Id=\"6\" Parameter=\"IsInternational\" Value=\"0\" Type=\"bool\" /><item Id=\"7\" Parameter=\"TypeWeapon\" Value=\"1\" Type=\"number\" /><item Id=\"8\" Parameter=\"StatisticsSendInterval\" Value=\"300\" Type=\"number\" /><item Id=\"9\" Parameter=\"SwfVersion\" Value=\"\" Type=\"string\" /><item Id=\"10\" Parameter=\"LanguageId\" Value=\"1\" Type=\"number\" /><item Id=\"11\" Parameter=\"SnId\" Value=\"1\" Type=\"number\" /><item Id=\"12\" Parameter=\"AutoServerSelectionAllowed\" Value=\"0\" Type=\"bool\" /></config>".getBytes())).toLowerCase() + "\" />\r\n" + 
    			"<cdata value=\""+ Util.bytesToHex(new RC4("_level0".getBytes()).encrypt(("<system ServerDate=\""+this.getDate()+"\" RPath=\"swf/cache/rus/resources[19].swf\" RVersion=\"1\" />").getBytes())).toLowerCase() +"\" />\r\n" + 
    			"<cdata value=\""+ Util.bytesToHex(new RC4("_level0".getBytes()).encrypt(("<user UserId=\""+UID+"\" hwId=\""+UUID.randomUUID().toString()+"\" ticketId=\""+UUID.randomUUID().toString()+"\" RoleFlags=\""+DB.getRoleFlags(UID)+"\" />").getBytes())).toLowerCase() +"\" />\r\n" + 
    			""+"<cdata value=\""+ Util.bytesToHex(new RC4("_level0".getBytes()).encrypt(("<servers><item Id=\"1\" TRId=\"202\" RId=\"5\" RTMPUrl=\"rtmp://localhost:"+this.rtmp+"/shararam\" Load=\""+String.valueOf(this.count)+"\" FriendsCount=\"1\" ClubsCount=\"5\" Weight=\"5.1\" /></servers>").getBytes())).toLowerCase() +"\" />"+"\r\n" + 
    			"</response>";
    	
    	return response; 
    }
    
    private String getDate() {
    	SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
    	Date date = new Date();
    	return format.format(date);
    }
}
