package org.cl0zzzy.rtmp;

import java.math.BigInteger;
import java.util.ArrayList;

public class AMFBuilder {
	
	private ArrayList<AMFObject> objs = new ArrayList<AMFObject>();
	
	private byte[] timestamp = new byte[] { 0x00, 0x00, 0x00 };
	
	private byte empty = 0x00;
	private byte ver = 0x03;
	private byte amf = 0x14;
	private byte _true = 0x01;
	private byte _false = 0x00;
	
	public AMFBuilder() {}
	
	public void setTimestamp( byte[] ts ) {
		this.timestamp = ts;
	}
	
	public void addAMFObject( AMFObject obj ) {
			this.objs.add(obj);
	}
	
	private ArrayList<Byte> encodeNumber( String str ) {
		ArrayList<Byte> result = new ArrayList<Byte>();
		byte[] arr = Util.hexStringToByteArray( this.doubleToNumber( Double.parseDouble(str) ) );
		for( int i = 0; i < arr.length; i++ ) {
			result.add(arr[i]);
		}
		return result;
	}
	
	private ArrayList<Byte> encodeString( String str ) {
		ArrayList<Byte> result = new ArrayList<Byte>();
		byte[] data = new byte[2];
		data[1] = (byte) (str.getBytes().length & 0xFF);
		data[0] = (byte) ((str.getBytes().length >> 8) & 0xFF);
		result.add(data[0]);
		result.add(data[1]);
		for( int i = 0; i < str.getBytes().length; i++ ) {
			result.add(str.getBytes()[i]);
		}
		return result;
	}
	
	private ArrayList<Byte> encodeBoolean( Boolean b ) {
		ArrayList<Byte> result = new ArrayList<Byte>();
		if( b ) {
			result.add(this._true);
		} else {
			result.add(this._false);
		}
		return result;
	}
	
	private ArrayList<Byte> encodeECMA( AMFEcmaArray ecma ) {
		ArrayList<Byte> result = new ArrayList<Byte>();
		result.add((byte)0x00);
		result.add((byte)0x00);
		result.add((byte)0x00);
		result.add((byte)0x00);
		for( int i = 0; i < ecma.getLength(); i++ ) {
			result.add((byte)0x00);
			result.add((byte) (Math.floor(i/10)+1));
			byte[] b = Util.hexStringToByteArray(toHex(String.valueOf(i)));
			for( int u = 0; u < b.length; u++ ) {
				result.add(b[u]);
			}
			ArrayList<Byte> _b = this.encodeAMFObject( ecma.getValueAt(i) );
			for( int u = 0; u < _b.size(); u++ ) {
				result.add(_b.get(u));
			}
		}
		result.add((byte)0x00);
		result.add((byte)0x00);
		result.add((byte)0x09);
		return result;
	}
	
	private ArrayList<Byte> encodeObj( AMFObjectArray arr ) {
		ArrayList<Byte> result = new ArrayList<Byte>();
		for( int i = 0; i < arr.getLength(); i++ ) {
			ArrayList<Byte> key = this.encodeString(arr.getKeyAt(i));
			ArrayList<Byte> body = this.encodeAMFObject(arr.getValueAt(i));
			for( int u = 0; u < key.size(); u++ ) {
				result.add(key.get(u));
			}
			result.add(arr.getValueAt(i).getType());
			for( int u = 0; u < body.size(); u++ ) {
				result.add(body.get(u));
			}
		}
		result.add((byte)0x00);
		result.add((byte)0x00);
		result.add(AMFObject.AMF_OBJECT_END);
		return result;
	}
	
	private ArrayList<Byte> encodeAMFObject( AMFObject obj ) {
		ArrayList<Byte> result = new ArrayList<Byte>();
		if( obj.getType() == AMFObject.AMF_NUMBER ) {
			result = this.encodeNumber( obj.getString() );
		} else if( obj.getType() == AMFObject.AMF_STRING ) {
			result = this.encodeString( obj.getString() );
		} else if( obj.getType() == AMFObject.AMF_BOOLEAN ) {
			result = this.encodeBoolean( obj.getBoolean() );
		} else if( obj.getType() == AMFObject.AMF_OBJECT_START ) {
			result = this.encodeObj( obj.getObjectArray() );
		} else if( obj.getType() == AMFObject.AMF_ECMA_ARRAY_START ) {
			result = this.encodeECMA( obj.getECMArray() );
		}
		return result;
	}
	
	public byte[] build() {
		ArrayList<Byte> array = new ArrayList<Byte>();
		array.add(ver);
		array.add(this.timestamp[0]);
		array.add(this.timestamp[1]);
		array.add(this.timestamp[2]);
		array.add(empty);
		array.add(empty);
		array.add(empty);
		array.add(amf);
		array.add(empty);
		array.add(empty);
		array.add(empty);
		array.add(empty);
		ArrayList<Byte> _array = new ArrayList<Byte>();
		for( int i = 0; i < this.objs.size(); i++ ) {
			AMFObject obj = this.objs.get(i);
			_array.add(obj.getType());
			ArrayList<Byte> nb = this.encodeAMFObject( obj );
			for( int u = 0; u < nb.size(); u++ ) {
				_array.add(nb.get(u));
			}
		}
		int cnt = 0;
		for( int i = 0; i < _array.size(); i++ ) {
			array.add(_array.get(i));
			if( i >= 4095 && ((i+1) % 4096 == 0) && (i+1) < _array.size() ) {
				array.add((byte) 0xC3);
				cnt++;
			}
		}
		int length = array.size() - 12 - cnt;
		byte[] ldata = new byte[3];
		ldata[2] = (byte) (length & 0xFF);
		ldata[1] = (byte) ((length >> 8) & 0xFF);
		ldata[0] = (byte) ((length >> 16) & 0xFF);
		array.set(4, ldata[0]);
		array.set(5, ldata[1]);
		array.set(6, ldata[2]);
		int j=0;
		byte[] _a = new byte[array.size()];
		for(Byte b : array) {
			_a[j++] = b.byteValue();
		}
		return _a;
	}
	
	private String doubleToNumber( double d ) {
		long longBits = Double.doubleToLongBits(d);
		if( longBits == 0 ) {
			return "0000000000000000";
		}
		return Long.toHexString(longBits).toUpperCase();
	}
	
	public String toHex(String arg) {
	    return String.format("%040x", new BigInteger(1, arg.getBytes()));
	}
}
