package org.cl0zzzy.rtmp;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class Chunk {
	
	private byte[] timestamp = new byte[0];
	private byte[] length = new byte[0];
	private byte[] amf;
	
	private byte type;
	
	public Chunk( byte[] array ) {
		type = array[0];
		if( array[0] == 0x03 ) {
			timestamp = Arrays.copyOfRange( array, 1, 4 );
			length = Arrays.copyOfRange( array, 4, 7 );
			amf = Arrays.copyOfRange( array, 12, 12+getDecimalLength() );
		} else if( array[0] == 0x43 ) {
			timestamp = Arrays.copyOfRange( array, 1, 4 );
			length = Arrays.copyOfRange( array, 4, 7 );
			amf = Arrays.copyOfRange( array, 8, 8+getDecimalLength() );
			if( 8+getDecimalLength() < array.length ) {
				amf = Arrays.copyOfRange( array, 8, array.length );
			}
		} else if( array[0] == (byte) 0x83 ) {
			timestamp = Arrays.copyOfRange( array, 1, 4 );
			amf = Arrays.copyOfRange( array, 4, array.length );
		}
		this.clearC3();
	}
	
	public Chunk( byte[] array, boolean ignore ) {
		type = array[0];
		if( array[0] == 0x03 ) {
			timestamp = Arrays.copyOfRange( array, 1, 4 );
			length = Arrays.copyOfRange( array, 4, 7 );
			amf = Arrays.copyOfRange( array, 12, array.length );
		} else if( array[0] == 0x43 ) {
			timestamp = Arrays.copyOfRange( array, 1, 4 );
			length = Arrays.copyOfRange( array, 4, 7 );
			amf = Arrays.copyOfRange( array, 8, array.length );
		} else if( array[0] == (byte) 0x83 ) {
			timestamp = Arrays.copyOfRange( array, 1, 4 );
			amf = Arrays.copyOfRange( array, 4, array.length );
		}
		this.clearC3();
	}
	
	public int getPos() {
		if( type == 0x03 ) {
			return 12;
		} else if( type == 0x43 ) {
			return 8;
		} else if( type == 0x83 ) {
			return 4;
		}
		return 0;
	}
	
	private void clearC3() {
		try {
			List<Byte> array = new ArrayList<Byte>();
			for( int i = 0; i < amf.length; i++ ) {
				array.add(amf[i]);
			}
			int every = 128;
			
		    for( int i = 1; i <= (int) (array.size()/every); i++ ) {
		    	if( Util.bytesToHex( new byte[] {array.get(i*every).byteValue()} ).equals(Util.bytesToHex(new byte[] {(byte) 0xC3})) ) {
		    		array.remove(i*every);
		    	}
		    }
	
			int j=0;
			amf = new byte[array.size()];
			for(Byte b : array)
			    amf[j++] = b.byteValue();
		} catch( Exception e ) {
			//e.printStackTrace();
		}
	}
	
	public int getDecimalLength() {
		return Integer.parseInt( Util.bytesToHex(this.length), 16 );
	}
	
	public byte[] getByteLength() {
		return this.length;
	}
	
	public byte[] getByteTimestamp() {
		return this.timestamp;
	}
	
	public byte[] getByteAMF() {
		return this.amf;
	}
	
	public static int getChunkLength( byte[] nb ) {
		byte[] length = Arrays.copyOfRange( nb, 4, 7 );
		int dl = Integer.parseInt( Util.bytesToHex(length), 16 );
		if( nb[0] == 0x03 ) {
			return 12 + dl;
		} else if( nb[0] == 0x43 ) {
			return 8 + dl;
		}
		return nb.length;
	}
	
	public static int getChunkLengthW( byte[] nb ) {
		byte[] length = Arrays.copyOfRange( nb, 4, 7 );
		int dl = Integer.parseInt( Util.bytesToHex(length), 16 );
		if( nb[0] == 0x03 ) {
			return dl;
		} else if( nb[0] == 0x43 ) {
			return dl;
		}
		return nb.length;
	}
	
	public static int getC3( int len ) {
		int cnt = 0;
		for( int i = 0; i < len; i++ ) {
			if( i >= 4095 && ((i+1) % 4096 == 0) && (i+1) < len ) {
				cnt++;
			}
		}
		return cnt;
	}
	
}
