package org.cl0zzzy.server;

import java.io.StringWriter;
import java.io.Writer;
import java.util.ArrayList;

import org.cl0zzzy.http.HttpMain;
import org.cl0zzzy.sqlite.DB;
import org.cl0zzzy.rtmp.AMFEcmaArray;
import org.cl0zzzy.rtmp.AMFObject;
import org.cl0zzzy.rtmp.AMFObjectArray;
import org.cl0zzzy.rtmp.RTMPServer;
import org.simpleframework.xml.Serializer;
import org.simpleframework.xml.core.Persister;

public class Server extends RTMPServer {
	
	private final Base Base         = new Base();
	private final Avatar Avatar     = new Avatar();
	private final Location Location = new Location();
	private final Home Home         = new Home();
	
	private ArrayList<Player> players = new ArrayList<Player>();
	
	private HttpMain HTTP;
	
	private String toXML( ParsableObject c ) {
		Writer writer = new StringWriter();
		Serializer serializer = new Persister();
		try {
		    serializer.write(c, writer);
		    return writer.toString();
		} catch (Exception e) {
		    e.printStackTrace();
		}
		return "<x/>";
	}

	public Server(int port, int httpPort) {
		super(port);
		org.cl0zzzy.sqlite.DB.connect();
		this.HTTP = new HttpMain(httpPort, port);
	}
	
	private Player getPlayerById( int id ) {
		for( int i = 0; i < players.size(); i++ ) {
			if( players.get(i).getId() == id ) {
				return players.get(i);
			}
		}
		return null;
	}
	
	private void sendInLoc( int location, String function, AMFObject... args ) {
		for( int i = 0; i < players.size(); i++ ) {
			if( players.get(i).getLocation() == (location) && players.get(i).isInLocation() ) {
				this.call(players.get(i).getId(), function, args);
			}
		}
	}
	
	public void onOpen( int id ) {
		System.out.println( "connection " + id );
		Player p = new Player(id);
		players.add(p);
		HTTP.updatePlayers( players.size() );
		DB.clearMagicAbilities( id );
	}
	
	public ArrayList<AMFObject> onMessage( int id, String function, ArrayList<AMFObject> data ) {
		ArrayList<AMFObject> result = new ArrayList<AMFObject>();
		
		if( !function.equals("_LG") && !function.equals("_S") && !data.get(1).getString().equals("_SR") && !function.equals("_SR") && !function.equals("_P") ) {
			System.out.println( "onMessage " + id + " " + function + " " + data.get(1).getString());
			
			for( int i = 0; i < data.size(); i++ ) {
				System.out.println( i + " " + data.get(i).getString() );
			}
		}
		
		if( function.equals("_S") ) {
			Base.setState( id, data.get(1).getString() );
		} else if( function.equals("_P") ) {
			Base.setPosition( id, data.get(1), data.get(2).getDouble() );
		} else if( function.equals("$") ) {
			if( data.get(1).getString().equals("_I") ) {
				result.add(new AMFObject( AMFObject.AMF_NULL, null ));
				System.out.println(Avatar.getUserInitData( this.getPlayerById(id) ) );
				result.add(new AMFObject( AMFObject.AMF_STRING, Avatar.getUserInitData( this.getPlayerById(id) ) ));
			} else if( data.get(1).getString().equals("_MC") ) {
				int recieverId = data.get(4).getECMArray().getValueAt(0).getInt();
				String cardId = data.get(4).getECMArray().getValueAt(1).getString();
				String pcid = DB.sendPostcard( id, recieverId, cardId );
				call( recieverId, "_MC", new AMFObject(AMFObject.AMF_NULL, null), new AMFObject(AMFObject.AMF_STRING, "<postcard><item Id=\""+pcid+"\" DateSent=\"\" SenderId=\""+id+"\" SenderName=\""+this.getPlayerById(id).getName()+"\" CardId=\""+cardId+"\" IsNew=\"1\" /></postcard>") );
				result.add(new AMFObject( AMFObject.AMF_NULL, null));
			} else if( data.get(1).getString().equals("_MP") ) {
				int recieverId = data.get(4).getECMArray().getValueAt(0).getInt();
				String message = data.get(4).getECMArray().getValueAt(1).getString();
				String msgId = DB.sendPhoneMessage( id, recieverId, message );
				call( recieverId, "_MP", new AMFObject(AMFObject.AMF_NULL, null),
						new AMFObject(AMFObject.AMF_STRING, "<item Id=\""+msgId+"\" DateSent=\"\" SenderId=\""+id+"\" SenderName=\""+this.getPlayerById(id).getName()+"\" Text=\""+message+"\" IsNew=\"1\" />") );
				result.add(new AMFObject( AMFObject.AMF_NULL, null));
			} else if( data.get(1).getString().equals("_MCD") ) {
				DB.removePostcard(data.get(4).getECMArray().getValueAt(0).getString());
				result.add(new AMFObject( AMFObject.AMF_NULL, null));
			} else if( data.get(1).getString().equals("_MCR") ) {
				DB.markPostcardAsRead( data.get(4).getECMArray().getValueAt(0).getString() );
				result.add(new AMFObject( AMFObject.AMF_NULL, null));
			} else if( data.get(1).getString().equals("_MPR") ) {
				DB.markMessageAsRead( data.get(4).getECMArray().getValueAt(0).getString() );
				result.add(new AMFObject( AMFObject.AMF_NULL, null));
			} else if( data.get(1).getString().equals("_SNFP") ) {
				result.add(new AMFObject( AMFObject.AMF_NULL, null ));
				result.add(new AMFObject( AMFObject.AMF_STRING, "" ));
			} else if( data.get(1).getString().equals("_D") ) {
				result.add(new AMFObject( AMFObject.AMF_NULL, null ));
				result.add(new AMFObject( AMFObject.AMF_NULL, null ));
			} else if( data.get(1).getString().equals("_CM") ) {
				result.add(new AMFObject( AMFObject.AMF_NULL, null ));
				result.add(new AMFObject( AMFObject.AMF_STRING, "</clubmap>" ));
			} else if( data.get(1).getString().equals("_LDH") ) {
				result.add(new AMFObject( AMFObject.AMF_NULL, null ));
				result.add(new AMFObject( AMFObject.AMF_STRING, "<root><inventory>"+DB.getFurnitureFromInventory(id)+"</inventory><locations><item ID=\"112\" MediaResourceID=\"1399\" IsHome=\"1\" IsCurrent=\"1\" IsLocked=\"1\" />"+DB.getHousesFromInventory(id)+"</locations></root>" ));
			} else if( data.get(1).getString().equals("_LG") ) {
				if( data.get(4).getECMArray().getValueAt(1).getDouble() == -1.0 ) {
					int owner = data.get(4).getECMArray().getValueAt(0).getInt();
					result = Home.getHome(result, owner, id);
				} else if( data.get(4).getECMArray().getValueAt(1).getDouble() == -2.0 ) {

				} else {
					result = Location.getLocation( this.getPlayerById(id), data.get(4).getECMArray().getValueAt(1).getDouble(), data.get(4).getECMArray().getValueAt(2).getDouble());
					this.getPlayerById(id).setX(Double.parseDouble(Location.getLocationX(data.get(4).getECMArray().getValueAt(1).getDouble(), data.get(4).getECMArray().getValueAt(2).getDouble())));
					this.getPlayerById(id).setY(Double.parseDouble(Location.getLocationY(data.get(4).getECMArray().getValueAt(1).getDouble(), data.get(4).getECMArray().getValueAt(2).getDouble())));
					Base.updateLocationPlayers(id, Location.getLocationId(data.get(4).getECMArray().getValueAt(1).getDouble(), data.get(4).getECMArray().getValueAt(2).getDouble()));
				}
			} else if( data.get(1).getString().equals("_LGI") ) {
				double nextLocation = data.get(4).getECMArray().getValueAt(0).getDouble();
				double prevLocation = data.get(4).getECMArray().getValueAt(1).getDouble();
				if( nextLocation == -1.0 ) {
					nextLocation = 0;
					prevLocation = 0;
				}
				result = Location.getLocation( this.getPlayerById(id), nextLocation, prevLocation);
				this.getPlayerById(id).setX(Double.parseDouble(Location.getLocationX(nextLocation, prevLocation)));
				this.getPlayerById(id).setY(Double.parseDouble(Location.getLocationY(nextLocation, prevLocation)));
				Base.updateLocationPlayers(id, Location.getLocationId(nextLocation, prevLocation));
			} else if( data.get(1).getString().equals("_UFG") ) {
				result.add(new AMFObject( AMFObject.AMF_NULL, null ));
				result.add(new AMFObject( AMFObject.AMF_OBJECT_START, DB.getUserFriends( id, players ) ));
			} else if( data.get(1).getString().equals("_UFR") ) {
				result.add(new AMFObject( AMFObject.AMF_NULL, null ));
				result.add(new AMFObject( AMFObject.AMF_OBJECT_START, DB.getUserFriendsRequests( id, players ) ));
			} else if( data.get(1).getString().equals("_UFO") ) {
				result.add(new AMFObject( AMFObject.AMF_NULL, null ));
				result.add(new AMFObject( AMFObject.AMF_OBJECT_START, DB.getUserFriends( id, players ) ));
			} else if( data.get(1).getString().equals("_UFA") ) {
				result.add(new AMFObject( AMFObject.AMF_NULL, null ));
				result.add(new AMFObject( AMFObject.AMF_STRING, "" ));
				String[] uids = new String[1];
				if( !data.get(4).getECMArray().getValueAt(1).getString().equals("") ) {
					uids = data.get(4).getECMArray().getValueAt(1).getString().split(",");
					for( int k = 0; k < uids.length; k++ ) {
						DB.removeFriend( id, Integer.parseInt(uids[k]) );
					}
				} else {
					uids = data.get(4).getECMArray().getValueAt(0).getString().split(",");
					for( int k = 0; k < uids.length; k++ ) {
						DB.addFriend( id, Integer.parseInt(uids[k]) );

						AMFObjectArray array = new AMFObjectArray();
						AMFObjectArray player = new AMFObjectArray();
						player.add("UserName", new AMFObject( AMFObject.AMF_STRING, DB.getPlayerName(Integer.parseInt(uids[k])) ));
						player.add("UserId", new AMFObject( AMFObject.AMF_STRING, uids[k] ));
						player.add("SnName", new AMFObject( AMFObject.AMF_STRING, "" ));
						boolean is = false;
						for( int i = 0; i < players.size(); i++ ) {
							if( players.get(i).getId() == Integer.parseInt(uids[k]) ) {
								is = true;
								break;
							}
						}
						if( is ) {
							player.add("IsOnline", new AMFObject( AMFObject.AMF_STRING, "1" ) );
						} else {
							player.add("IsOnline", new AMFObject( AMFObject.AMF_STRING, "0" ) );
						}
						array.add("1", new AMFObject( AMFObject.AMF_OBJECT_START, player ));

						call( id, "_UFLA", new AMFObject( AMFObject.AMF_NULL, null ),
								new AMFObject( AMFObject.AMF_OBJECT_START, array ) );

						array = new AMFObjectArray();
						player = new AMFObjectArray();
						player.add("UserName", new AMFObject( AMFObject.AMF_STRING, this.getPlayerById(id).getName() ));
						player.add("UserId", new AMFObject( AMFObject.AMF_STRING, id ));
						player.add("SnName", new AMFObject( AMFObject.AMF_STRING, "" ));
						is = false;
						for( int i = 0; i < players.size(); i++ ) {
							if( players.get(i).getId() == id ) {
								is = true;
								break;
							}
						}
						if( is ) {
							player.add("IsOnline", new AMFObject( AMFObject.AMF_STRING, "1" ) );
						} else {
							player.add("IsOnline", new AMFObject( AMFObject.AMF_STRING, "0" ) );
						}
						array.add("1", new AMFObject( AMFObject.AMF_OBJECT_START, player ));

						call( Integer.parseInt(uids[k]), "_UFLA", new AMFObject( AMFObject.AMF_NULL, null ),
								new AMFObject( AMFObject.AMF_OBJECT_START, array ) );
					}
				}
			} else if( data.get(1).getString().equals("_UFD") ) {
				result.add(new AMFObject( AMFObject.AMF_NULL, null ));
				result.add(new AMFObject( AMFObject.AMF_STRING, "" ));
				DB.removeFriend( id, data.get(4).getECMArray().getValueAt(0).getInt() );
			} else if( data.get(1).getString().equals("_NUM") ) {
				Base.useMagicAbility( data.get(4).getECMArray().getValueAt(0).getInt(), data.get(4).getECMArray().getValueAt(1).getDouble() );
			} else if( data.get(1).getString().equals("_NUS") ) {
				if( data.get(2).getString().equals("ROOM") ) {
					Base.throwSnowball( data.get(4).getECMArray().getValueAt(0).getInt(), data.get(4).getECMArray().getValueAt(1).getDouble(), data.get(4).getECMArray().getValueAt(2).getDouble() );
				}
			} else if( data.get(1).getString().equals("_C") ) {
				if( data.get(4).getECMArray().getValueAt(1).getString().startsWith("/give ") ) {
					try {
						DB.addPlayerCloth(id, Integer.parseInt(data.get(4).getECMArray().getValueAt(1).getString().substring(6)) + "");
					} catch( Exception e ) {
						e.printStackTrace();
					}
				}
				Base.chatMessage( id, data.get(4).getECMArray().getValueAt(1).getString() );
			} else if( data.get(1).getString().equals("_UUG") ) {
				result.add(new AMFObject( AMFObject.AMF_NULL, null ));
				result.add(new AMFObject( AMFObject.AMF_STRING, "123" ));
			} else if( data.get(1).getString().equals("_GG") ) {
				result.add(new AMFObject( AMFObject.AMF_NULL, null ));
				result.add(new AMFObject( AMFObject.AMF_STRING, DB.getGameXML(data.get(4).getECMArray().getValueAt(0).getString()) ));
				System.out.println( DB.getGameXML(data.get(4).getECMArray().getValueAt(0).getString()) );
			} else if( data.get(1).getString().equals("_GC") ) {
				result.add(new AMFObject( AMFObject.AMF_NULL, null ));
				result.add(new AMFObject( AMFObject.AMF_STRING, "0" ));
				getPlayerById(id).setInLocation(true);
			} else if( data.get(1).getString().equals("_GD") ) {
				this.getPlayerById(id).setInMultiplayerGame(false);
			} else if( data.get(1).getString().equals("_GR") ) {
				result.add(new AMFObject( AMFObject.AMF_NULL, null ));

				this.getPlayerById(id).setInMultiplayerGame(true);

				AMFObjectArray array = new AMFObjectArray();
				array.add("ID", new AMFObject( AMFObject.AMF_STRING, id ));
				array.add("Name", new AMFObject( AMFObject.AMF_STRING, this.getPlayerById(id).getName() ));

				AMFObjectArray ecma = new AMFObjectArray();
				ecma.add("0", new AMFObject(AMFObject.AMF_OBJECT_START, array));

				array = new AMFObjectArray();
				array.add("ID", new AMFObject( AMFObject.AMF_STRING, "2" ));
				array.add("Name", new AMFObject( AMFObject.AMF_STRING, "Натик" ));

				ecma.add("1", new AMFObject(AMFObject.AMF_OBJECT_START, array));

				AMFObjectArray _arr = new AMFObjectArray();
				_arr.add("0", new AMFObject(AMFObject.AMF_OBJECT_START, ecma));

				new Thread( () -> { try { Thread.sleep(2000); } catch( Exception e ) { e.printStackTrace(); } call( id, "_GU", new AMFObject( AMFObject.AMF_NULL, null ), new AMFObject( AMFObject.AMF_OBJECT_START, _arr ),
						new AMFObject( AMFObject.AMF_NUMBER, 1 ), new AMFObject( AMFObject.AMF_BOOLEAN, false ) ); } ).start();

				new Thread( () -> {  try { Thread.sleep(2500); } catch( Exception e ) { e.printStackTrace(); } call( id, "_GL", new AMFObject( AMFObject.AMF_NULL, null ), new AMFObject( AMFObject.AMF_OBJECT_START, _arr ),
						new AMFObject( AMFObject.AMF_STRING, "222" )); } ).start();

				new Thread( () -> {
					try {
						Thread.sleep(3000);
						int i = 0;
						while( i < 100 ) {
							call(id, "_G", new AMFObject( AMFObject.AMF_NULL, null ), new AMFObject( AMFObject.AMF_STRING, "OnReady" ), new AMFObject( AMFObject.AMF_NUMBER, 0 ), new AMFObject( AMFObject.AMF_STRING, id ));
							i++;
							Thread.sleep(500);
						}
					} catch ( Exception e ) {
						e.printStackTrace();
					}
				}).start();
			} else if( data.get(1).getString().equals("_GT") ) {
				result.add(new AMFObject( AMFObject.AMF_NULL, null ));
				result.add(new AMFObject( AMFObject.AMF_STRING, "0" ));
			} else if( data.get(1).getString().equals("_GS") ) {
				result.add(new AMFObject( AMFObject.AMF_NULL, null ));
				Base.setMoney(id, DB.getPlayerUsualTickets(id)+data.get(4).getECMArray().getValueAt(1).getInt(), DB.getPlayerMagicTickets(id));
				result.add(new AMFObject( AMFObject.AMF_STRING, "" ));
			} else if( data.get(1).getString().equals("_AG") ) {
				result.add(new AMFObject( AMFObject.AMF_NULL, null ));
				result.add(new AMFObject( AMFObject.AMF_STRING, Avatar.getAG(id) ));
			} else if( data.get(1).getString().equals("_AS") ) {
				result.add(new AMFObject( AMFObject.AMF_NULL, null ));
				Avatar.setAG(id, data.get(4).getECMArray());
				result.add(new AMFObject( AMFObject.AMF_ECMA_ARRAY_START, new AMFEcmaArray() ));
				this.sendInLoc(this.getPlayerById(id).getLocation(), "_UL", new AMFObject( AMFObject.AMF_NULL, null ), new AMFObject( AMFObject.AMF_STRING, id ));
				this.sendInLoc(this.getPlayerById(id).getLocation(), "_UE", new AMFObject( AMFObject.AMF_NULL, null ), new AMFObject( AMFObject.AMF_STRING, id ), new AMFObject( AMFObject.AMF_OBJECT_START, Avatar.getUE(id) ) );
			} else if( data.get(1).getString().equals("_UI") ) {
				AMFObjectArray friends = DB.getUserFriends(id, players);
				String isFriend = "0";
				for( int i = 0; i < friends.getLength(); i++ ) {
					String user = friends.getValueAt(i).getObjectArray().get("UserId").getString();
					String uid = String.valueOf(((int)Double.parseDouble(data.get(4).getECMArray().getValueAt(0).getString())));
					if( uid.equals(user) ) {
						isFriend = "1";
						break;
					}
				}
				result.add(new AMFObject( AMFObject.AMF_NULL, null ));
				result.add(new AMFObject( AMFObject.AMF_STRING, Avatar.getUI(data.get(4).getECMArray().getValueAt(0).getInt(), isFriend) ));
			} else if( data.get(1).getString().equals("_GCI") ) {
				result.add(new AMFObject( AMFObject.AMF_NULL, null ));
				result.add(new AMFObject( AMFObject.AMF_STRING, this.toXML(DB.getCatalogById(data.get(4).getECMArray().getValueAt(0).getString())) ));
			} else if( data.get(1).getString().equals("_JD") ) {
				result.add(new AMFObject( AMFObject.AMF_NULL, null ));
				result.add(new AMFObject( AMFObject.AMF_STRING, "<garage><Car><item Id=\"8156304\" CarTypeId=\"1\" MediaResourceID=\"2135\" IsCurrent=\"1\" /></Car><Inventory><item Id=\"75\" TypeId=\"75\" CarTypeId=\"1\" MediaResourceID=\"2235\"><item Id=\"59497575\" UserCarId=\"8156304\" /></item><item Id=\"76\" TypeId=\"77\" CarTypeId=\"1\" MediaResourceID=\"2131\"><item Id=\"59497576\" UserCarId=\"8156304\" /></item><item Id=\"77\" TypeId=\"72\" CarTypeId=\"1\" MediaResourceID=\"2127\"><item Id=\"59497577\" UserCarId=\"8156304\" /></item><item Id=\"78\" TypeId=\"74\" CarTypeId=\"1\" MediaResourceID=\"2236\"><item Id=\"59497578\" UserCarId=\"8156304\" /></item><item Id=\"79\" TypeId=\"73\" CarTypeId=\"1\" MediaResourceID=\"2237\"><item Id=\"59497579\" UserCarId=\"8156304\" /></item><item Id=\"80\" TypeId=\"76\" CarTypeId=\"1\" MediaResourceID=\"2238\"><item Id=\"59497580\" UserCarId=\"8156304\" /></item><item Id=\"81\" TypeId=\"81\" CarTypeId=\"1\" MediaResourceID=\"2239\"><item Id=\"59497581\" UserCarId=\"8156304\" /></item></Inventory></garage>"));
			} else if( data.get(1).getString().equals("_VC") ) {
				result.add(new AMFObject( AMFObject.AMF_NULL, null ));
				result.add(new AMFObject( AMFObject.AMF_BOOLEAN, true ));
			} else if( data.get(1).getString().equals("_BGB") ) {
				result.add(new AMFObject( AMFObject.AMF_NULL, null ));
				AMFObjectArray good = data.get(4).getECMArray().getValueAt(0).getECMArray().getValueAt(0).getObjectArray().get("item").getObjectArray();
				if( good.get("priceSm").getDouble() <= DB.getPlayerUsualTickets(id) &&
						good.get("priceRu").getDouble() <= DB.getPlayerMagicTickets(id) ) {
					AMFObjectArray tags = good.get("tagsHash").getObjectArray();
					if( tags.get("cloth") != null || tags.get("costume") != null || tags.get("back") != null ) {
						DB.addPlayerCloth( id, good.get("id").getString() );
					} else if( tags.get("magic") != null ) {
						if( good.get("magicAbilityTime").getString().equals("null") ) {
							DB.addMagicAbility(id, good.get("magicAbilityId").getString(), "0", "0");
						} else {
							DB.addMagicAbility(id, good.get("magicAbilityId").getString(), good.get("magicAbilityTime").getString(), "1");
						}
					} else if( tags.get("furniture") != null ) {
						DB.addPlayerFurniture( id, good.get("id").getString() );
					} else if( tags.get("house") != null ) {
						DB.addPlayerHome( id, good.get("id").getString() );
					} else {
						//System.out.println( "cloth == null" );
						//System.out.println( tags.toString() );
					}
					Base.setMoney(id, DB.getPlayerUsualTickets(id)-good.get("priceSm").getInt(), DB.getPlayerMagicTickets(id)-good.get("priceRu").getInt());
					AMFObjectArray arr = new AMFObjectArray();
					arr.add("Error", new AMFObject( AMFObject.AMF_BOOLEAN, false ));
					arr.add("GoodInfo", new AMFObject( AMFObject.AMF_ECMA_ARRAY_START, new AMFEcmaArray() ));
					result.add(new AMFObject( AMFObject.AMF_OBJECT_START, arr ));
				} else {
					result.add(new AMFObject( AMFObject.AMF_STRING, "0" ) );
				}
			} else if( data.get(1).getString().equals("_UFQ") ) {
				result.add(new AMFObject( AMFObject.AMF_NULL, null ));
				result.add(new AMFObject( AMFObject.AMF_STRING, "" ));
				AMFObjectArray player = new AMFObjectArray();
				player.add("UserName", new AMFObject( AMFObject.AMF_STRING, DB.getPlayerName(id) ));
				player.add("UserId", new AMFObject( AMFObject.AMF_STRING, id ));
				player.add("SnName", new AMFObject( AMFObject.AMF_STRING, "" ));
				player.add("IsOnline", new AMFObject( AMFObject.AMF_STRING, "1" ) );

				call(data.get(4).getECMArray().getValueAt(0).getInt(), "_UFLR",
						new AMFObject( AMFObject.AMF_NULL, null ),
						new AMFObject( AMFObject.AMF_OBJECT_START, player));
				DB.requestFriend(id, data.get(4).getECMArray().getValueAt(0).getInt());
			} else if( data.get(1).getString().equals("_LM") ) {
				result.add(new AMFObject( AMFObject.AMF_NULL, null ));
				result.add(new AMFObject( AMFObject.AMF_STRING, DB.getUserMagicAbilities(id) ));
			} else if( data.get(1).getString().equals("_GMB") ) {
				result.add(new AMFObject( AMFObject.AMF_NULL, null ));
				AMFObjectArray array = new AMFObjectArray();
				array.add("Amount", new AMFObject( AMFObject.AMF_NUMBER, 9999 ));
				result.add(new AMFObject( AMFObject.AMF_OBJECT_START, array ));
			} else if( data.get(1).getString().equals("_MTB") ) {
				result.add(new AMFObject( AMFObject.AMF_NULL, null ));
				AMFObjectArray array = new AMFObjectArray();
				array.add("Amount", new AMFObject( AMFObject.AMF_NUMBER, 9999 ));
				array.add("Success", new AMFObject( AMFObject.AMF_NUMBER, 1 ));
				result.add(new AMFObject( AMFObject.AMF_OBJECT_START, array ));
				Base.setMoney(id,  DB.getPlayerUsualTickets(id),DB.getPlayerMagicTickets(id)+100);
			} else if( data.get(1).getString().equals("_TE") ) {
				int magic_rem = 0;
				int usual_add = 0;
				if( data.get(4).getECMArray().getValueAt(0).getDouble() == 1.0 ) {
					magic_rem = 2;
					usual_add = 100;
				} else if( data.get(4).getECMArray().getValueAt(0).getDouble() == 2.0 ) {
					magic_rem = 4;
					usual_add = 200;
				} else if( data.get(4).getECMArray().getValueAt(0).getDouble() == 3.0 ) {
					magic_rem = 10;
					usual_add = 500;
				} else if( data.get(4).getECMArray().getValueAt(0).getDouble() == 4.0 ) {
					magic_rem = 20;
					usual_add = 1000;
				} else if( data.get(4).getECMArray().getValueAt(0).getDouble() == 5.0 ) {
					magic_rem = 50;
					usual_add = 2500;
				}
				result.add(new AMFObject( AMFObject.AMF_NULL, null ));
				AMFObjectArray array = new AMFObjectArray();
				array.add("Success", new AMFObject( AMFObject.AMF_NUMBER, 1 ));
				result.add(new AMFObject( AMFObject.AMF_OBJECT_START, array ));
				Base.setMoney(id, DB.getPlayerUsualTickets(id)+usual_add, DB.getPlayerMagicTickets(id)-magic_rem);
			} else if( data.get(1).getString().equals("_LC") ) {
				if( data.get(2).getString().equals("ROOM") ) {
					String type = data.get(4).getECMArray().getValueAt(1).getString();
					if( type.equals("-1.0") ) {
						AMFObjectArray object = data.get(4).getECMArray().getValueAt(2).getObjectArray();
						AMFObjectArray Add = object.get("Add").getObjectArray();
						AMFObjectArray Remove = object.get("Remove").getObjectArray();
						//System.out.println("Add + " + Add.toString());
						String xml = "";
						for( int i = 0; i < Add.getLength(); i++ ) {
							String key = Add.getKeyAt(i);
							AMFObjectArray oa = Add.get(key).getObjectArray();
							xml += "<f ";
							for( int u = 0; u < oa.getLength(); u++ ) {
								xml += oa.getKeyAt(u) + "=\""+oa.getValueAt(u)+"\" ";
							}
							xml += "/>";
						}
						//System.out.println( xml );
						//System.out.println("Remove + " + Remove.toString());
					}
				}
			} else if( data.get(1).getString().equals("_CUR") ) {
				result.add(new AMFObject( AMFObject.AMF_NULL, null ) );
				result.add(new AMFObject( AMFObject.AMF_STRING, this.toXML(new CheckUserRightsObject(false, false, DB.getGoodById((int) data.get(4).getECMArray().getValueAt(0).getDouble()).getTextResourceID(), DB.getGoodById((int) data.get(4).getECMArray().getValueAt(0).getDouble()).getMediaResourceID())) ));
			}
		} else if( function.equals("_LS") ) {
			if( data.get(1).getString().startsWith("G$") ) {
				getPlayerById(id).setInLocation(false);
				Base.deleteSelf(id, this.getPlayerById(id).getLocation());
			} else {
				if( this.getPlayerById(id).isInLocation() ) {
					Base.updateLocationPlayers(id, this.getPlayerById(id).getLocation());
					sendInLoc(this.getPlayerById(id).getLocation(), "_UE", new AMFObject( AMFObject.AMF_NULL, null ), new AMFObject( AMFObject.AMF_STRING, id ), new AMFObject( AMFObject.AMF_OBJECT_START, Avatar.getUE(id) ));
				}
			}
			result.add(new AMFObject( AMFObject.AMF_NULL, null ));
			result.add(new AMFObject( AMFObject.AMF_STRING, "" ));
		} else if( function.equals("_LG") ) {

			//System.out.println( " data " );

			/*for( AMFObject d : data ) {
				System.out.println( d.toString() );
			}*/

			result.add(new AMFObject( AMFObject.AMF_NULL, null ));
			result.add(new AMFObject( AMFObject.AMF_STRING, "222" ));
			result.add(new AMFObject( AMFObject.AMF_STRING, data.get(1).getString() ));
		}
		return result;
	}

	public void onClose( int id ) {
		System.out.println( "onClose " + id );
		for( int i = 0; i < players.size(); i++ ) {
			if( players.get(i).getId() == id ) {
				this.broadcast("_UL", new AMFObject( AMFObject.AMF_NULL, null ), new AMFObject( AMFObject.AMF_STRING, id ));
				this.players.remove(i);
				break;
			}
		}
		HTTP.updatePlayers( players.size() );
	}

	private class Base {

		private class stateThread extends Thread {

			private int id;
			private Player p;

			public stateThread( int id, Player p ) {
				this.p = p;
				this.id = id;
			}

			@Override
			public void run() {
				try { Thread.sleep(800); } catch( Exception e ) { e.printStackTrace(); }
				setState( p.getId(), String.valueOf( p.getState() ), id );
			}
		}

		public void setMoney( int id, int usual, int magic ) {
			call( id, "_NNT", new AMFObject( AMFObject.AMF_NULL, null ),
					new AMFObject(AMFObject.AMF_NUMBER, usual),
					new AMFObject(AMFObject.AMF_NUMBER, magic ) );
			DB.updateMoney( id, usual, magic );
		}

		private void setState( int id, String state, int sendId ) {
			Player p = getPlayerById(id);
			getPlayerById(id).setState(Double.parseDouble( state ));
			if( p.isInLocation() ) {
				call( sendId, "_S", new AMFObject( AMFObject.AMF_NULL, null ), new AMFObject( AMFObject.AMF_STRING, id ), new AMFObject( AMFObject.AMF_NUMBER, state ));
			}
		}

		public void deleteSelf( int id, int location ) {
			sendInLoc( location, "_UL", new AMFObject( AMFObject.AMF_NULL, null ), new AMFObject( AMFObject.AMF_STRING, id ) );
		}

		private void sendSelf( int id, int location ) {
			sendInLoc( location, "_UE",
					new AMFObject( AMFObject.AMF_NULL, null ),
					new AMFObject( AMFObject.AMF_STRING, id ),
					new AMFObject( AMFObject.AMF_OBJECT_START, Avatar.getUE(id)));
		}

		public void updateLocationPlayers( int id, int location ) {
			if( location != getPlayerById(id).getLocation() ) {
				this.deleteSelf(id, getPlayerById(id).getLocation());
				for( Player p : players ) {
					if( p.getLocation() == getPlayerById(id).getLocation() ) {
						call( id, "_UL", new AMFObject( AMFObject.AMF_NULL, null ),
								new AMFObject( AMFObject.AMF_STRING, p.getId() ) );
					}
				}
			}
			getPlayerById(id).setInLocation(true);
			getPlayerById(id).setLocation(location);
			this.sendSelf(id, location);
			call( id, "_UE", new AMFObject( AMFObject.AMF_NULL, null ),
					new AMFObject( AMFObject.AMF_STRING, id ),
					new AMFObject( AMFObject.AMF_OBJECT_START, Avatar.getUE(id) ));
			for( Player p : players ) {
				if( p.getId() != id && p.getLocation() == location && p.isInLocation() ) {
					call( id, "_UE", new AMFObject( AMFObject.AMF_NULL, null ),
							new AMFObject( AMFObject.AMF_STRING, p.getId() ),
							new AMFObject( AMFObject.AMF_OBJECT_START, Avatar.getUE(p.getId()) ));
					new stateThread( id, p ).start();
				}
			}
		}

		public void setPosition( int id, AMFObject obj, double num ) {
			AMFObjectArray arr = new AMFObjectArray();
			if( obj.getObjectArray() != null ) {
				arr.add("x", obj.getObjectArray().get("x"));
				arr.add("y", obj.getObjectArray().get("y"));
			} else {
				arr.add("x", new AMFObject( AMFObject.AMF_NUMBER, getPlayerById(id).getX() ) );
				arr.add("y", new AMFObject( AMFObject.AMF_NUMBER, getPlayerById(id).getY() ) );
			}
			getPlayerById(id).setCoords(arr.get("x").getDouble(), arr.get("y").getDouble());
			if( getPlayerById(id).isInLocation() ) {
				sendInLoc(getPlayerById(id).getLocation(), "_P", new AMFObject( AMFObject.AMF_NULL, null ), new AMFObject( AMFObject.AMF_STRING, id ), new AMFObject( AMFObject.AMF_OBJECT_START,  arr ), new AMFObject( AMFObject.AMF_NUMBER, num ));
			}
		}

		public void setState( int id, String state ) {
			Player p = getPlayerById(id);
			getPlayerById(id).setState(Double.parseDouble( state ));
			if( p.isInLocation() ) {
				sendInLoc(p.getLocation(), "_S", new AMFObject( AMFObject.AMF_NULL, null ), new AMFObject( AMFObject.AMF_STRING, id ), new AMFObject( AMFObject.AMF_NUMBER, state ));
			}
		}

		public void throwSnowball( int id, double x, double y ) {
			if( getPlayerById(id).isInLocation() ) {
				sendInLoc(getPlayerById(id).getLocation(), "_NUS", new AMFObject( AMFObject.AMF_NULL, null ), new AMFObject( AMFObject.AMF_STRING, id ), new AMFObject( AMFObject.AMF_NUMBER, x ), new AMFObject( AMFObject.AMF_NUMBER, y ) );
			}
		}

		public void useMagicAbility( int id, double ability ) {
			if( getPlayerById(id).isInLocation() ) {
				sendInLoc(getPlayerById(id).getLocation(), "_NUM", new AMFObject( AMFObject.AMF_NULL, null ), new AMFObject( AMFObject.AMF_STRING, id ), new AMFObject( AMFObject.AMF_NUMBER, ability  ) );
			}
		}

		public void chatMessage( int id, String msg ) {
			if( getPlayerById(id).isInLocation() ) {
				sendInLoc(getPlayerById(id).getLocation(), "_C", new AMFObject( AMFObject.AMF_NULL, null ), new AMFObject( AMFObject.AMF_STRING, id ), new AMFObject( AMFObject.AMF_STRING, msg ));
			}
		}

	}

	private class Avatar {

		private String getAG( int id ) {
			ArrayList<InventoryItem> bodyparts = new ArrayList<InventoryItem>();
			bodyparts.add(new InventoryItem(56, null, 0, null, 56, null, null, 56, 1, 16762375, null));
			bodyparts.add(new InventoryItem(60, null, 0, null, 60, null, null, 60, 1, 16762375, null));
			bodyparts.add(new InventoryItem(64, null, 0, null, 64, null, null, 64, 1, 16762375, null));
			bodyparts.add(new InventoryItem(69, null, 0, null, 69, null, null, 69, 1, 16762375, null));
			bodyparts.add(new InventoryItem(73, null, 0, null, 73, null, null, 73, 1, 16762375, null));
			bodyparts.add(new InventoryItem(77, null, 0, null, 77, null, null, 77, 1, 16762375, null));
			bodyparts.add(new InventoryItem(83, null, 0, null, 83, null, null, 83, 1, 16762375, null));
			bodyparts.add(new InventoryItem(89, null, 1, null, 89, null, null, 89, 1, 16762375, null));
			ArrayList<InventoryItem> legparts = new ArrayList<InventoryItem>();
			legparts.add(new InventoryItem(55, 0, 0, "", 55, 0, 0, 55, 1, 16762375, 1));
			legparts.add(new InventoryItem(62, 0, 0, "", 62, 0, 0, 62, 1, 16762375, 1));
			legparts.add(new InventoryItem(66, 0, 0, "", 66, 0, 0, 66, 1, 16762375, 1));
			legparts.add(new InventoryItem(71, 0, 0, "", 71, 0, 0, 71, 1, 16762375, 1));
			legparts.add(new InventoryItem(75, 0, 0, "", 75, 0, 0, 75, 1, 16762375, 1));
			legparts.add(new InventoryItem(80, 0, 1, "", 80, 0, 0, 80, 1, 16762375, 1));
			legparts.add(new InventoryItem(86, 0, 0, "", 86, 0, 0, 86, 1, 16762375, 1));
			legparts.add(new InventoryItem(92, 0, 0, "", 92, 0, 0, 92, 1, 16762375, 1));
			ArrayList<InventoryItem> earparts = new ArrayList<InventoryItem>();
			earparts.add(new InventoryItem(78, 0, 1, "", 78, 0, 0, 78, 1, 0, 1));
			earparts.add(new InventoryItem(84, 0, 0, "", 84, 0, 0, 84, 1, 16762375, 1));
			earparts.add(new InventoryItem(90, 0, 0, "", 90, 0, 0, 90, 1, 16762375, 1));
			earparts.add(new InventoryItem(95, 0, 0, "", 95, 0, 0, 95, 1, 16762375, 1));
			earparts.add(new InventoryItem(96, 0, 0, "", 96, 0, 0, 96, 1, 16762375, 1));
			earparts.add(new InventoryItem(97, 0, 0, "", 97, 0, 0, 97, 1, 16762375, 1));
			earparts.add(new InventoryItem(98, 0, 0, "", 98, 0, 0, 98, 1, 16762375, 1));
			ArrayList<InventoryItem> eyeparts = new ArrayList<InventoryItem>();
			eyeparts.add(new InventoryItem(59, 0, 1, "", 59, 0, 0, 59, 0, 1, 1));
			eyeparts.add(new InventoryItem(61, 0, 0, "", 61, 0, 0, 61, 0, 1, 1));
			eyeparts.add(new InventoryItem(65, 0, 0, "", 65, 0, 0, 65, 0, 1, 1));
			eyeparts.add(new InventoryItem(70, 0, 0, "", 70, 0, 0, 70, 0, 1, 1));
			eyeparts.add(new InventoryItem(74, 0, 0, "", 74, 0, 0, 74, 0, 1, 1));
			eyeparts.add(new InventoryItem(79, 0, 0, "", 79, 0, 0, 79, 0, 1, 1));
			eyeparts.add(new InventoryItem(85, 0, 0, "", 85, 0, 0, 85, 0, 1, 1));
			eyeparts.add(new InventoryItem(91, 0, 0, "", 91, 0, 0, 91, 0, 1, 1));
			ArrayList<InventoryItem> noseparts = new ArrayList<InventoryItem>();
			noseparts.add(new InventoryItem(58, 0, 1, "", 58, 0, 0, 58, 0, 1, 1));
			noseparts.add(new InventoryItem(68, 0, 0, "", 68, 0, 0, 68, 0, 1, 1));
			noseparts.add(new InventoryItem(82, 0, 0, "", 82, 0, 0, 82, 0, 1, 1));
			noseparts.add(new InventoryItem(88, 0, 0, "", 88, 0, 0, 88, 0, 1, 1));
			noseparts.add(new InventoryItem(94, 0, 0, "", 94, 0, 0, 94, 0, 1, 1));
			noseparts.add(new InventoryItem(98, 0, 0, "", 98, 0, 0, 98, 0, 1, 1));
			ArrayList<InventoryItem> mouthparts = new ArrayList<InventoryItem>();
			mouthparts.add(new InventoryItem(57, 0, 1, "", 57, 0, 0, 57, 0, 1, 1));
			mouthparts.add(new InventoryItem(67, 0, 0, "", 67, 0, 0, 67, 0, 1, 1));
			mouthparts.add(new InventoryItem(81, 0, 0, "", 81, 0, 0, 81, 0, 1, 1));
			mouthparts.add(new InventoryItem(87, 0, 0, "", 87, 0, 0, 87, 0, 1, 1));
			mouthparts.add(new InventoryItem(93, 0, 0, "", 93, 0, 0, 93, 0, 1, 1));
			ArrayList<InventoryItem> beakparts = new ArrayList<InventoryItem>();
			beakparts.add(new InventoryItem(63, 0, 1, "", 63, 0, 0, 63, 0, 1, 1));
			beakparts.add(new InventoryItem(72, 0, 0, "", 72, 0, 0, 72, 0, 1, 1));
			beakparts.add(new InventoryItem(76, 0, 0, "", 76, 0, 0, 76, 0, 1, 1));
			beakparts.add(new InventoryItem(98, 0, 0, "", 98, 0, 0, 98, 0, 1, 1));
			ArrayList<InventoryItem> hornparts = new ArrayList<InventoryItem>();
			hornparts.add(new InventoryItem(99, 0, 1, "", 99, 0, 0, 99, 0, 1, 1));
			hornparts.add(new InventoryItem(100, 0, 0, "", 100, 0, 0, 100, 0, 1, 1));
			hornparts.add(new InventoryItem(101, 0, 0, "", 101, 0, 0, 101, 0, 1, 1));
			hornparts.add(new InventoryItem(102, 0, 0, "", 102, 0, 0, 102, 0, 1, 1));
			hornparts.add(new InventoryItem(103, 0, 0, "", 103, 0, 0, 103, 0, 1, 1));
			hornparts.add(new InventoryItem(98, 0, 0, "", 98, 0, 0, 98, 0, 1, 1));
			InventoryGroup body = new InventoryGroup(3, 48, 0, 202, 20, bodyparts);
			InventoryGroup legs = new InventoryGroup(2, 47, 0, 201, 10, legparts);
			InventoryGroup ears = new InventoryGroup(8, 70, 0, 206, 50, earparts);
			InventoryGroup eyes = new InventoryGroup(7, 58, 0, 205, 30, eyeparts);
			InventoryGroup nose = new InventoryGroup(6, 57, 0, 204, 40, noseparts);
			InventoryGroup mouth = new InventoryGroup(5, 56, 0, 203, 26, mouthparts);
			InventoryGroup beak = new InventoryGroup(11, 110, 0, 208, 41, beakparts);
			InventoryGroup horn = new InventoryGroup(12, 166, 0, 209, 55, hornparts);
			InventoryGroup clothes = new InventoryGroup(1, 120, 0, 212, 36, DB.getGoodsFromInventory(id, "1"));
			InventoryGroup hats = new InventoryGroup(4, 21, 0, 211, 45, DB.getGoodsFromInventory(id, "4"));
			InventoryGroup glasses = new InventoryGroup(14, 62, 0, 210, 31, DB.getGoodsFromInventory(id, "14"));
			InventoryGroup shoes = new InventoryGroup(32, 123, 0, 213, 11, DB.getGoodsFromInventory(id, "32"));
			InventoryGroup misc = new InventoryGroup(36, 24, 0, 214, 56, DB.getGoodsFromInventory(id, "36"));
			InventoryGroup costumes = new InventoryGroup(40, 697, 0, 429, 27, DB.getGoodsFromInventory(id, "40"));
			ArrayList<InventoryGroup> invtab = new ArrayList<InventoryGroup>();
			invtab.add(clothes);
			invtab.add(hats);
			invtab.add(glasses);
			invtab.add(shoes);
			invtab.add(misc);
			invtab.add(costumes);
			ArrayList<InventoryGroup> shape = new ArrayList<InventoryGroup>();
			shape.add(body);
			shape.add(legs);
			ArrayList<InventoryGroup> bparts = new ArrayList<InventoryGroup>();
			bparts.add(ears);
			bparts.add(eyes);
			bparts.add(nose);
			bparts.add(mouth);
			bparts.add(beak);
			bparts.add(horn);
			for (InventoryGroup inventoryGroup : shape) {
				for (int j = 0; j < inventoryGroup.getItems().size(); j++) {
					inventoryGroup.getItems().get(j).setIsUsed(DB.getIsUsedGood(id, inventoryGroup.getItems().get(j)));
					inventoryGroup.getItems().get(j).setColor(DB.getBodyPartColor(id, inventoryGroup.getItems().get(j)));
				}
			}
			for (InventoryGroup bpart : bparts) {
				for (int j = 0; j < bpart.getItems().size(); j++) {
					bpart.getItems().get(j).setIsUsed(DB.getIsUsedGood(id, bpart.getItems().get(j)));
					bpart.getItems().get(j).setColor(DB.getBodyPartColor(id, bpart.getItems().get(j)));
				}
			}
			for (InventoryGroup bpart : invtab) {
				for (int j = 0; j < bpart.getItems().size(); j++) {
					bpart.getItems().get(j).setIsUsed(DB.getIsUsedGood(id, bpart.getItems().get(j)));
				}
			}
			Inventory inv = new Inventory(
					new InventoryTab( 199, shape ),
					new InventoryTab( 205, bparts ),
					new InventoryTab( 206, invtab ),
					new InventoryList( 230, new ArrayList<InventoryItem>()),
					new InventoryList( 231, new ArrayList<InventoryItem>()),
					new InventoryList( 211, new ArrayList<InventoryItem>()));
			return toXML(inv).replace("<user_avatar>", "").replace("</user_avatar>", "");
		}

		private void setAG( int id, AMFEcmaArray arr ) {
			DB.clearUserdata(id);
			AMFObjectArray array = arr.getValueAt(0).getObjectArray();
			for( int i = 0; i < array.get("BodyParts").getECMArray().getLength(); i++ ) {
				AMFObjectArray obj = array.get("BodyParts").getECMArray().getValueAt(i).getObjectArray();
				String color = obj.get("Color").getString().equals("NaN") ? "-1" : obj.get("Color").getString();
				if( !obj.get("Id").getString().equals("98.0") ) {
					DB.insertUserdata(id,
							obj.get("Id").getString(), "1", obj.get("Id").getString(), color, "0");
				}
			}
			for( int i = 0; i < array.get("Inventory").getECMArray().getLength(); i++ ) {
				AMFObjectArray obj = array.get("Inventory").getECMArray().getValueAt(i).getObjectArray();
				DB.insertUserdata(id,
						obj.get("Id").getString(), "0", "0", "-1", obj.get("Id").getString());
			}
		}

		private String getUserInitData( Player p ) {
			p.setName( DB.getPlayerName(p.getId()) );
			UserMagicAbilities ma = DB.getUserMagicAbilities(p.getId());
			UserAccount ua = DB.getUserAccount(p.getId());
			UserInfo ui = DB.getUserInfo(p.getId(), 0);
			UserPhone up = DB.getUserPhone(p.getId());
			ArrayList<InventoryItem> av = DB.getUserAvatar(p.getId());
			UserInitInfo init = new UserInitInfo( ma, ua, ui, up, av );
			return toXML(init);
		}

		private AMFObjectArray getUE( int id ) {
			AMFObjectArray array = new AMFObjectArray();
			array.add("RoleFlags", new AMFObject( AMFObject.AMF_STRING, "2" ));
			AMFObjectArray user = new AMFObjectArray();
			user.add("Level", new AMFObject( AMFObject.AMF_STRING, "99" ));
			user.add("Name", new AMFObject( AMFObject.AMF_STRING, getPlayerById(id).getName() ));
			user.add("x", new AMFObject( AMFObject.AMF_STRING, getPlayerById(id).getX() ));
			user.add("y", new AMFObject( AMFObject.AMF_STRING, getPlayerById(id).getY() ));
			user.add("State", new AMFObject( AMFObject.AMF_NUMBER, String.valueOf( getPlayerById(id).getState() )));
			AMFObjectArray body = DB.getUserData(id);
			user.add("Body", new AMFObject( AMFObject.AMF_OBJECT_START, body ));
			array.add("User", new AMFObject( AMFObject.AMF_OBJECT_START, user ));
			return array;
		}

		private String getUI( int id, String isFriend ) {
			//String avatar = DB.getUserDataXML( id );
			String avatar = "";
			return "<user UserId=\""+id+"\" Name=\""+DB.getPlayerName(id)+"\" RoleFlags=\""+DB.getPlayerRoleFlags(id)+"\" DateRegistered=\"2008-08-08T12:07:44.927\" IsLimited=\"0\" IsOnline=\"1\" IsMinorSecretAgent=\""+DB.getPlayerIsMinorSecretAgent(id)+"\" IsHomeLocked=\"1\" IsClubLocked=\""+DB.getPlayerIsClubLocked(id)+"\" IsClubPresent=\""+DB.getPlayerIsClubPresent(id)+"\" ClubAccessType=\"0\" IsFriend=\""+isFriend+"\" Experience=\""+DB.getPlayerExperience(id)+"\" Level=\""+DB.getPlayerLevel(id)+"\" PositiveEnergy=\"6\" MagicLevel=\""+DB.getPlayerMagicLevel(id)+"\" RaceLevel=\"0\" PhoneID=\""+DB.getPlayerPhoneID(id)+"\" BackgroundID=\""+DB.getPlayerBackgroundID(id)+"\" CampId=\"0\" NiftTotalCount=\"0\" LocationServerName=\"Неофициальный сервер\"><ExtendedInfo UserId=\""+id+"\" HasKey=\"0\" /><SpecialOffers><Offer Id=\"5002\" MinsLeft=\"494483\" ActionTypeId=\"0\" IconMRId=\"27214\" PopupMRId=\"27215\" /><Offer Id=\"10006\" MinsLeft=\"-396877\" ActionTypeId=\"3\" IconMRId=\"14831\" PopupMRId=\"14832\" /></SpecialOffers></user><avatar>"+avatar+"</avatar>";
		}

	}

	private class Home {
		public ArrayList<AMFObject> getHome( ArrayList<AMFObject> result, int owner, int id ) {
			result.add(new AMFObject( AMFObject.AMF_NULL, null ));
			result.add(new AMFObject( AMFObject.AMF_STRING, "<location ID=\"-1\" OwnerId=\""+owner+"\" OwnerName=\""+DB.getPlayerName(owner)+"\" IsHome=\"1\" IsClub=\"0\" IsLocked=\"\" IsVisaAccessPermitted=\"1\" MediaResourceID=\"708\" x=\"650.0\" y=\"520.0\"></location>"));
			System.out.println( result.get(result.size()-1).getString() );
			Base.updateLocationPlayers(id, -1);
			return result;
		}
	}

	private class Location {
		public ArrayList<AMFObject> getLocation( Player p, double id, double _id ) {
			ArrayList<AMFObject> result = new ArrayList<AMFObject>();
			result.add(new AMFObject( AMFObject.AMF_NULL, null ));
			if( id == 0.0 && _id == 0.0 ) {
				if( p.getCanModifyLocations() ) {
					result.add(new AMFObject( AMFObject.AMF_STRING, org.cl0zzzy.sqlite.DB.getDefaultLocationXML().replace("IsHome=\"0\"", "IsHome=\"1\"") ));
				} else {
					result.add(new AMFObject( AMFObject.AMF_STRING, org.cl0zzzy.sqlite.DB.getDefaultLocationXML() ));
				}
			} else {
				if( p.getCanModifyLocations() ) {
					result.add(new AMFObject( AMFObject.AMF_STRING, org.cl0zzzy.sqlite.DB.getLocationXML((int)(id)).replace("IsHome=\"0\"", "IsHome=\"1\"") ));
				} else {
					result.add(new AMFObject( AMFObject.AMF_STRING, org.cl0zzzy.sqlite.DB.getLocationXML((int)(id)) ));
				}
			}
			return result;
		}
		public int getLocationId( double id, double _id ) {
			if( id == 0.0 && _id == 0.0 ) {
				return org.cl0zzzy.sqlite.DB.getDefaultLocationId();
			} else {
				return org.cl0zzzy.sqlite.DB.getLocationId((int)(id));
			}
		}
		public String getLocationX( double id, double _id ) {
			if( id == 0.0 && _id == 0.0 ) {
				return org.cl0zzzy.sqlite.DB.getDefaultLocationX();
			} else {
				return org.cl0zzzy.sqlite.DB.getLocationX((int) (id));
			}
		}
		public String getLocationY( double id, double _id ) {
			if( id == 0.0 && _id == 0.0 ) {
				return org.cl0zzzy.sqlite.DB.getDefaultLocationY();
			} else {
				return org.cl0zzzy.sqlite.DB.getLocationY((int)(id));
			}
		}
	}

}
